@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id: llrasterize.cpp 33715 2016-03-13 08:52:06Z goatbar $
  *
  * Project:  GDAL
  * Purpose:  Vector polygon rasterization code.
@@ -28,19 +27,23 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "gdal_alg.h"
+#include "cpl_port.h"
 #include "gdal_alg_priv.h"
 
+#include <cmath>
+#include <cstdlib>
+#include <cstring>
+
+#include <algorithm>
+#include <utility>
+
+#include "gdal_alg.h"
+
+CPL_CVSID("$Id: llrasterize.cpp 41013 2017-12-12 12:05:53Z rouault $");
+
 static int llCompareInt(const void *a, const void *b)
 {
-	return (*(const int *)a) - (*(const int *)b);
-}
-
-static void llSwapDouble(double *a, double *b)
-{
-	double temp = *a;
-	*a = *b;
-	*b = temp;
+    return (*(const int *)a) - (*(const int *)b);
 }
 
 /************************************************************************/
@@ -97,142 +100,149 @@ void GDALdllImageFilledPolygon(int nRasterXSize, int nRasterYSize,
 No known bug
 *************************************************************************/
 
-    int i;
-    int y;
-    int miny, maxy,minx,maxx;
-    double dminy, dmaxy;
-    double dx1, dy1;
-    double dx2, dy2;
-    double dy;
-    double intersect;
-
-    int ind1, ind2;
-    int ints, n, part;
-    int *polyInts;
-
-    int horizontal_x1, horizontal_x2;
-
-    if (!nPartCount) {
+    if( !nPartCount )
+    {
         return;
     }
 
-    n = 0;
-    for( part = 0; part < nPartCount; part++ )
+    int n = 0;
+    for( int part = 0; part < nPartCount; part++ )
         n += panPartSize[part];
 
-    // +1 to make clang static analyzer not warn about potential malloc(0)
-    polyInts = (int *) malloc(sizeof(int) * (n+1));
+    // +1 to make clang static analyzer not warn about potential malloc(0).
+    int *polyInts = static_cast<int *>(malloc(sizeof(int) * (n + 1)));
 
-    dminy = padfY[0];
-    dmaxy = padfY[0];
-    for (i=1; (i < n); i++) {
-
-        if (padfY[i] < dminy) {
+    double dminy = padfY[0];
+    double dmaxy = padfY[0];
+    for( int i = 1; i < n; i++ )
+    {
+        if( padfY[i] < dminy )
+        {
             dminy = padfY[i];
         }
-        if (padfY[i] > dmaxy) {
+        if( padfY[i] > dmaxy )
+        {
             dmaxy = padfY[i];
         }
     }
-    miny = (int) dminy;
-    maxy = (int) dmaxy;
+    int miny = static_cast<int>(dminy);
+    int maxy = static_cast<int>(dmaxy);
 
     if( miny < 0 )
         miny = 0;
     if( maxy >= nRasterYSize )
-        maxy = nRasterYSize-1;
+        maxy = nRasterYSize - 1;
 
-    minx = 0;
-    maxx = nRasterXSize - 1;
+    int minx = 0;
+    const int maxx = nRasterXSize - 1;
 
-    /* Fix in 1.3: count a vertex only once */
-    for (y=miny; y <= maxy; y++) {
-        int	partoffset = 0;
+    // Fix in 1.3: count a vertex only once.
+    for( int y = miny; y <= maxy; y++ )
+    {
+        int partoffset = 0;
 
-        dy = y +0.5; /* center height of line*/
+        const double dy = y + 0.5;  // Center height of line.
 
-        part = 0;
-        ints = 0;
+        int part = 0;
+        int ints = 0;
 
-        /*Initialize polyInts, otherwise it can sometimes causes a seg fault */
+        // Initialize polyInts, otherwise it can sometimes causes a seg fault.
         memset(polyInts, -1, sizeof(int) * n);
 
-        for (i=0; (i < n); i++) {
-
-            if( i == partoffset + panPartSize[part] ) {
+        for( int i = 0; i < n; i++ )
+        {
+            if( i == partoffset + panPartSize[part] )
+            {
                 partoffset += panPartSize[part];
                 part++;
             }
 
-            if( i == partoffset ) {
+            int ind1 = 0;
+            int ind2 = 0;
+            if( i == partoffset )
+            {
                 ind1 = partoffset + panPartSize[part] - 1;
                 ind2 = partoffset;
-            } else {
+            }
+            else
+            {
                 ind1 = i-1;
                 ind2 = i;
             }
 
-            dy1 = padfY[ind1];
-            dy2 = padfY[ind2];
+            double dy1 = padfY[ind1];
+            double dy2 = padfY[ind2];
 
             if( (dy1 < dy && dy2 < dy) || (dy1 > dy && dy2 > dy) )
                 continue;
 
-            if (dy1 < dy2) {
+            double dx1 = 0.0;
+            double dx2 = 0.0;
+            if( dy1 < dy2 )
+            {
                 dx1 = padfX[ind1];
                 dx2 = padfX[ind2];
-            } else if (dy1 > dy2) {
+            }
+            else if( dy1 > dy2 )
+            {
                 dy2 = padfY[ind1];
                 dy1 = padfY[ind2];
                 dx2 = padfX[ind1];
                 dx1 = padfX[ind2];
-            } else /* if (fabs(dy1-dy2)< 1.e-6) */
+            }
+            else // if( fabs(dy1-dy2) < 1.e-6 )
             {
 
-                /*AE: DO NOT skip bottom horizontal segments
-                  -Fill them separately-
-                  They are not taken into account twice.*/
-                if (padfX[ind1] > padfX[ind2])
+                // AE: DO NOT skip bottom horizontal segments
+                // -Fill them separately-
+                // They are not taken into account twice.
+                if( padfX[ind1] > padfX[ind2] )
                 {
-                    horizontal_x1 = (int) floor(padfX[ind2]+0.5);
-                    horizontal_x2 = (int) floor(padfX[ind1]+0.5);
+                    const int horizontal_x1 =
+                        static_cast<int>(floor(padfX[ind2] + 0.5));
+                    const int horizontal_x2 =
+                        static_cast<int>(floor(padfX[ind1] + 0.5));
 
-                    if  ( (horizontal_x1 >  maxx) ||  (horizontal_x2 <= minx) )
+                    if( (horizontal_x1 >  maxx) ||  (horizontal_x2 <= minx) )
                         continue;
 
-                    /*fill the horizontal segment (separately from the rest)*/
-                    pfnScanlineFunc( pCBData, y, horizontal_x1, horizontal_x2 - 1, (dfVariant == NULL)?0:dfVariant[0] );
+                    // Fill the horizontal segment (separately from the rest).
+                    pfnScanlineFunc( pCBData, y, horizontal_x1,
+                                     horizontal_x2 - 1,
+                                     (dfVariant == NULL)?0:dfVariant[0] );
                     continue;
                 }
-                else /*skip top horizontal segments (they are already filled in the regular loop)*/
+                else
+                {
+                    // Skip top horizontal segments (they are already filled in
+                    // the regular loop).
                     continue;
-
+                }
             }
 
-            if(( dy < dy2 ) && (dy >= dy1))
+            if( dy < dy2 && dy >= dy1 )
             {
+                const double intersect = (dy-dy1) * (dx2-dx1) / (dy2-dy1) + dx1;
 
-                intersect = (dy-dy1) * (dx2-dx1) / (dy2-dy1) + dx1;
-
-                polyInts[ints++] = (int) floor(intersect+0.5);
+                polyInts[ints++] = static_cast<int>(floor(intersect + 0.5));
             }
         }
 
-        /*
-         * It would be more efficient to do this inline, to avoid
-         * a function call for each comparison.
-         * NOTE - mloskot: make llCompareInt a functor and use std
-         * algorithm and it will be optimized and expanded
-         * automatically in compile-time, with modularity preserved.
-         */
+        // It would be more efficient to do this inline, to avoid
+        // a function call for each comparison.
+        // NOTE - mloskot: make llCompareInt a functor and use std
+        // algorithm and it will be optimized and expanded
+        // automatically in compile-time, with modularity preserved.
+        //
+        // TODO(schwehr): Use std::sort.
         qsort(polyInts, ints, sizeof(int), llCompareInt);
 
-
-        for (i=0; (i < (ints)); i+=2)
+        for( int i = 0; i < ints; i += 2 )
         {
             if( polyInts[i] <= maxx && polyInts[i+1] > minx )
             {
-                pfnScanlineFunc( pCBData, y, polyInts[i], polyInts[i+1] - 1, (dfVariant == NULL)?0:dfVariant[0] );
+                pfnScanlineFunc(pCBData, y, polyInts[i], polyInts[i+1] - 1,
+                                dfVariant == NULL ? 0 : dfVariant[0]);
             }
         }
     }
@@ -250,17 +260,15 @@ void GDALdllImagePoint( int nRasterXSize, int nRasterYSize,
                         double *padfX, double *padfY, double *padfVariant,
                         llPointFunc pfnPointFunc, void *pCBData )
 {
-    int i;
-
-    for ( i = 0; i < nPartCount; i++ )
+    for( int i = 0; i < nPartCount; i++ )
     {
-        int nX = (int)floor( padfX[i] );
-        int nY = (int)floor( padfY[i] );
-        double dfVariant = 0;
+        const int nX = static_cast<int>(floor( padfX[i] ));
+        const int nY = static_cast<int>(floor( padfY[i] ));
+        double dfVariant = 0.0;
         if( padfVariant != NULL )
             dfVariant = padfVariant[i];
 
-        if ( 0 <= nX && nX < nRasterXSize && 0 <= nY && nY < nRasterYSize )
+        if( 0 <= nX && nX < nRasterXSize && 0 <= nY && nY < nRasterYSize )
             pfnPointFunc( pCBData, nY, nX, dfVariant );
     }
 }
@@ -274,24 +282,21 @@ void GDALdllImageLine( int nRasterXSize, int nRasterYSize,
                        double *padfX, double *padfY, double *padfVariant,
                        llPointFunc pfnPointFunc, void *pCBData )
 {
-    int     i, n;
-
-    if ( !nPartCount )
+    if( !nPartCount )
         return;
 
-    for ( i = 0, n = 0; i < nPartCount; n += panPartSize[i++] )
+    for( int i = 0, n = 0; i < nPartCount; n += panPartSize[i++] )
     {
-        int j;
-
-        for ( j = 1; j < panPartSize[i]; j++ )
+        for( int j = 1; j < panPartSize[i]; j++ )
         {
-            int iX = (int)floor( padfX[n + j - 1] );
-            int iY = (int)floor( padfY[n + j - 1] );
+            int iX = static_cast<int>(floor( padfX[n + j - 1] ));
+            int iY = static_cast<int>(floor( padfY[n + j - 1] ));
 
-            const int iX1 = (int)floor( padfX[n + j] );
-            const int iY1 = (int)floor( padfY[n + j] );
+            const int iX1 = static_cast<int>(floor(padfX[n + j]));
+            const int iY1 = static_cast<int>(floor(padfY[n + j]));
 
-            double dfVariant = 0, dfVariant1 = 0;
+            double dfVariant = 0.0;
+            double dfVariant1 = 0.0;
             if( padfVariant != NULL &&
                 ((GDALRasterizeInfo *)pCBData)->eBurnValueSource !=
                     GBV_UserBurnValue )
@@ -300,39 +305,44 @@ void GDALdllImageLine( int nRasterXSize, int nRasterYSize,
                 dfVariant1 = padfVariant[n + j];
             }
 
-            int nDeltaX = ABS( iX1 - iX );
-            int nDeltaY = ABS( iY1 - iY );
+            int nDeltaX = std::abs(iX1 - iX);
+            int nDeltaY = std::abs(iY1 - iY);
 
             // Step direction depends on line direction.
             const int nXStep = ( iX > iX1 ) ? -1 : 1;
             const int nYStep = ( iY > iY1 ) ? -1 : 1;
 
             // Determine the line slope.
-            if ( nDeltaX >= nDeltaY )
+            if( nDeltaX >= nDeltaY )
             {
                 const int nXError = nDeltaY << 1;
                 const int nYError = nXError - (nDeltaX << 1);
                 int nError = nXError - nDeltaX;
-                /* == 0 makes clang -fcatch-undefined-behavior -ftrapv happy, but if */
-                /* it is == 0, dfDeltaVariant is not really used, so any value is OK */
-                double dfDeltaVariant = (nDeltaX == 0) ? 0 : (dfVariant1 - dfVariant) /
-                                                           (double)nDeltaX;
+                // == 0 makes clang -fcatch-undefined-behavior -ftrapv happy,
+                // but if it is == 0, dfDeltaVariant is not really used, so any
+                // value is okay.
+                const double dfDeltaVariant =
+                    nDeltaX == 0
+                    ? 0.0
+                    : (dfVariant1 - dfVariant) / static_cast<double>(nDeltaX);
 
-                while ( nDeltaX-- >= 0 )
+                while( nDeltaX-- >= 0 )
                 {
-                    if ( 0 <= iX && iX < nRasterXSize
-                         && 0 <= iY && iY < nRasterYSize )
+                    if( 0 <= iX && iX < nRasterXSize
+                        && 0 <= iY && iY < nRasterYSize )
                         pfnPointFunc( pCBData, iY, iX, dfVariant );
 
                     dfVariant += dfDeltaVariant;
                     iX += nXStep;
-                    if ( nError > 0 )
+                    if( nError > 0 )
                     {
                         iY += nYStep;
                         nError += nYError;
                     }
                     else
+                    {
                         nError += nXError;
+                    }
                 }
             }
             else
@@ -340,26 +350,31 @@ void GDALdllImageLine( int nRasterXSize, int nRasterYSize,
                 const int nXError = nDeltaX << 1;
                 const int nYError = nXError - (nDeltaY << 1);
                 int nError = nXError - nDeltaY;
-                /* == 0 makes clang -fcatch-undefined-behavior -ftrapv happy, but if */
-                /* it is == 0, dfDeltaVariant is not really used, so any value is OK */
-                double dfDeltaVariant = (nDeltaY == 0) ? 0 : (dfVariant1 - dfVariant) /
-                                                           (double)nDeltaY;
+                // == 0 makes clang -fcatch-undefined-behavior -ftrapv happy,
+                // but if it is == 0, dfDeltaVariant is not really used, so any
+                // value is okay.
+                double dfDeltaVariant =
+                    nDeltaY == 0
+                    ? 0.0
+                    : (dfVariant1 - dfVariant) / static_cast<double>(nDeltaY);
 
-                while ( nDeltaY-- >= 0 )
+                while( nDeltaY-- >= 0 )
                 {
-                    if ( 0 <= iX && iX < nRasterXSize
-                         && 0 <= iY && iY < nRasterYSize )
+                    if( 0 <= iX && iX < nRasterXSize
+                        && 0 <= iY && iY < nRasterYSize )
                         pfnPointFunc( pCBData, iY, iX, dfVariant );
 
                     dfVariant += dfDeltaVariant;
                     iY += nYStep;
-                    if ( nError > 0 )
+                    if( nError > 0 )
                     {
                         iX += nXStep;
                         nError += nYError;
                     }
                     else
+                    {
                         nError += nXError;
+                    }
                 }
             }
         }
@@ -380,22 +395,18 @@ void GDALdllImageLine( int nRasterXSize, int nRasterYSize,
 /************************************************************************/
 
 void
-GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
-                           int nPartCount, int *panPartSize,
-                           double *padfX, double *padfY, double *padfVariant,
-                           llPointFunc pfnPointFunc, void *pCBData )
+GDALdllImageLineAllTouched( int nRasterXSize, int nRasterYSize,
+                            int nPartCount, int *panPartSize,
+                            double *padfX, double *padfY, double *padfVariant,
+                            llPointFunc pfnPointFunc, void *pCBData )
 
 {
-    int     i, n;
-
-    if ( !nPartCount )
+    if( !nPartCount )
         return;
 
-    for ( i = 0, n = 0; i < nPartCount; n += panPartSize[i++] )
+    for( int i = 0, n = 0; i < nPartCount; n += panPartSize[i++] )
     {
-        int j;
-
-        for ( j = 1; j < panPartSize[i]; j++ )
+        for( int j = 1; j < panPartSize[i]; j++ )
         {
             double dfX = padfX[n + j - 1];
             double dfY = padfY[n + j - 1];
@@ -403,7 +414,8 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
             double dfXEnd = padfX[n + j];
             double dfYEnd = padfY[n + j];
 
-            double dfVariant = 0, dfVariantEnd = 0;
+            double dfVariant = 0.0;
+            double dfVariantEnd = 0.0;
             if( padfVariant != NULL &&
                 ((GDALRasterizeInfo *)pCBData)->eBurnValueSource !=
                     GBV_UserBurnValue )
@@ -413,47 +425,48 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
             }
 
             // Skip segments that are off the target region.
-            if( (dfY < 0 && dfYEnd < 0)
+            if( (dfY < 0.0 && dfYEnd < 0.0)
                 || (dfY > nRasterYSize && dfYEnd > nRasterYSize)
-                || (dfX < 0 && dfXEnd < 0)
+                || (dfX < 0.0 && dfXEnd < 0.0)
                 || (dfX > nRasterXSize && dfXEnd > nRasterXSize) )
                 continue;
 
             // Swap if needed so we can proceed from left2right (X increasing)
             if( dfX > dfXEnd )
             {
-                llSwapDouble( &dfX, &dfXEnd );
-                llSwapDouble( &dfY, &dfYEnd );
-                llSwapDouble( &dfVariant, &dfVariantEnd );
+                std::swap(dfX, dfXEnd);
+                std::swap(dfY, dfYEnd );
+                std::swap(dfVariant, dfVariantEnd);
             }
 
             // Special case for vertical lines.
-            if( floor(dfX) == floor(dfXEnd) )
+            if( floor(dfX) == floor(dfXEnd) || fabs(dfX - dfXEnd) < .01 )
             {
                 if( dfYEnd < dfY )
                 {
-                    llSwapDouble( &dfY, &dfYEnd );
-                    llSwapDouble( &dfVariant, &dfVariantEnd );
+                    std::swap(dfY, dfYEnd );
+                    std::swap(dfVariant, dfVariantEnd);
                 }
 
-                int iX = (int) floor(dfX);
-                int iY = (int) floor(dfY);
+                const int iX = static_cast<int>(floor(dfXEnd));
+                int iY = static_cast<int>(floor(dfY));
                 int iYEnd = (int) floor(dfYEnd);
 
-                if( iX >= nRasterXSize )
+                if( iX < 0 || iX >= nRasterXSize )
                     continue;
 
-                double dfDeltaVariant = 0;
-                if(( dfYEnd - dfY ) > 0)
-                    dfDeltaVariant = ( dfVariantEnd - dfVariant )
-                                     / ( dfYEnd - dfY );//per unit change in iY
+                double dfDeltaVariant = 0.0;
+                if( dfYEnd - dfY > 0.0 )
+                    dfDeltaVariant =
+                        ( dfVariantEnd - dfVariant ) /
+                        ( dfYEnd - dfY );  // Per unit change in iY.
 
-                // Clip to the borders of the target region
+                // Clip to the borders of the target region.
                 if( iY < 0 )
                     iY = 0;
                 if( iYEnd >= nRasterYSize )
                     iYEnd = nRasterYSize - 1;
-                dfVariant += dfDeltaVariant * ( (double)iY - dfY );
+                dfVariant += dfDeltaVariant * (static_cast<double>(iY) - dfY);
 
                 if( padfVariant == NULL )
                     for( ; iY <= iYEnd; iY++ )
@@ -462,34 +475,35 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
                     for( ; iY <= iYEnd; iY++, dfVariant +=  dfDeltaVariant )
                         pfnPointFunc( pCBData, iY, iX, dfVariant );
 
-                continue; // next segment
+                continue;  // Next segment.
             }
 
-            double dfDeltaVariant = ( dfVariantEnd - dfVariant )
-                                    / ( dfXEnd - dfX );//per unit change in iX
+            const double dfDeltaVariant =
+                ( dfVariantEnd - dfVariant ) /
+                ( dfXEnd - dfX );  // Per unit change in iX.
 
-            // special case for horizontal lines
-            if( floor(dfY) == floor(dfYEnd) )
+            // Special case for horizontal lines.
+            if( floor(dfY) == floor(dfYEnd) || fabs(dfY - dfYEnd) < .01 )
             {
                 if( dfXEnd < dfX )
                 {
-                    llSwapDouble( &dfX, &dfXEnd );
-                    llSwapDouble( &dfVariant, &dfVariantEnd );
+                    std::swap(dfX, dfXEnd);
+                    std::swap(dfVariant, dfVariantEnd);
                 }
 
-                int iX = (int) floor(dfX);
-                int iY = (int) floor(dfY);
+                int iX = static_cast<int>(floor(dfX));
+                const int iY = static_cast<int>(floor(dfY));
                 int iXEnd = (int) floor(dfXEnd);
 
-                if( iY >= nRasterYSize )
+                if( iY < 0 || iY >= nRasterYSize )
                     continue;
 
-                // Clip to the borders of the target region
+                // Clip to the borders of the target region.
                 if( iX < 0 )
                     iX = 0;
                 if( iXEnd >= nRasterXSize )
                     iXEnd = nRasterXSize - 1;
-                dfVariant += dfDeltaVariant * ( (double)iX - dfX );
+                dfVariant += dfDeltaVariant * (static_cast<double>(iX) - dfX);
 
                 if( padfVariant == NULL )
                     for( ; iX <= iXEnd; iX++ )
@@ -498,63 +512,71 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
                     for( ; iX <= iXEnd; iX++, dfVariant +=  dfDeltaVariant )
                         pfnPointFunc( pCBData, iY, iX, dfVariant );
 
-                continue; // next segment
+                continue;  // Next segment.
             }
 
 /* -------------------------------------------------------------------- */
 /*      General case - left to right sloped.                            */
 /* -------------------------------------------------------------------- */
-            double dfSlope = (dfYEnd - dfY) / (dfXEnd - dfX);
+            const double dfSlope = (dfYEnd - dfY) / (dfXEnd - dfX);
 
-            // clip segment in X
+            // Clip segment in X.
             if( dfXEnd > nRasterXSize )
             {
-                dfYEnd -= ( dfXEnd - (double)nRasterXSize ) * dfSlope;
+                dfYEnd -=
+                    (dfXEnd - static_cast<double>(nRasterXSize)) * dfSlope;
                 dfXEnd = nRasterXSize;
             }
-            if( dfX < 0 )
+            if( dfX < 0.0 )
             {
-                dfY += (0 - dfX) * dfSlope;
+                dfY += (0.0 - dfX) * dfSlope;
                 dfVariant += dfDeltaVariant * (0.0 - dfX);
                 dfX = 0.0;
             }
 
-            // clip segment in Y
-            double dfDiffX;
+            // Clip segment in Y.
+            double dfDiffX = 0.0;
             if( dfYEnd > dfY )
             {
-                if( dfY < 0 )
+                if( dfY < 0.0 )
                 {
-                    dfX += (dfDiffX = (0 - dfY) / dfSlope);
+                    dfDiffX = (0.0 - dfY) / dfSlope;
+                    dfX += dfDiffX;
                     dfVariant += dfDeltaVariant * dfDiffX;
                     dfY = 0.0;
                 }
                 if( dfYEnd >= nRasterYSize )
                 {
-                    dfXEnd += ( dfYEnd - (double)nRasterYSize ) / dfSlope;
-                    dfYEnd = nRasterXSize;
+                    dfXEnd +=
+                        (dfYEnd - static_cast<double>(nRasterYSize)) / dfSlope;
+                    // dfYEnd is no longer used afterwards, but for
+                    // consistency it should be:
+                    // dfYEnd = nRasterXSize;
                 }
             }
             else
             {
                 if( dfY >= nRasterYSize )
                 {
-                    dfX += (dfDiffX = ((double)nRasterYSize - dfY) / dfSlope);
+                  dfDiffX = (static_cast<double>(nRasterYSize) - dfY) / dfSlope;
+                    dfX += dfDiffX;
                     dfVariant += dfDeltaVariant * dfDiffX;
                     dfY = nRasterYSize;
                 }
-                if( dfYEnd < 0 )
+                if( dfYEnd < 0.0 )
                 {
                     dfXEnd -= ( dfYEnd - 0 ) / dfSlope;
-                    dfYEnd = 0;
+                    // dfYEnd is no longer used afterwards, but for
+                    // consistency it should be:
+                    // dfYEnd = 0.0;
                 }
             }
 
-            // step from pixel to pixel.
-            while( dfX >= 0 && dfX < dfXEnd )
+            // Step from pixel to pixel.
+            while( dfX >= 0.0 && dfX < dfXEnd )
             {
-                int iX = (int) floor(dfX);
-                int iY = (int) floor(dfY);
+                const int iX = static_cast<int>(floor(dfX));
+                const int iY = static_cast<int>(floor(dfY));
 
                 // Burn in the current point.
                 // We should be able to drop the Y check because we clipped
@@ -565,8 +587,8 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
                 double dfStepX = floor(dfX+1.0) - dfX;
                 double dfStepY = dfStepX * dfSlope;
 
-                // step to right pixel without changing scanline?
-                if( (int) floor(dfY + dfStepY) == iY )
+                // Step to right pixel without changing scanline?
+                if( static_cast<int>(floor(dfY + dfStepY)) == iY )
                 {
                     dfX += dfStepX;
                     dfY += dfStepY;
@@ -594,8 +616,7 @@ GDALdllImageLineAllTouched(int nRasterXSize, int nRasterYSize,
                     dfY += dfStepY;
                     dfVariant += dfDeltaVariant * dfStepX;
                 }
-            } // next step along segment.
-
-        } // next segment
-    } // next part
+            }  // Next step along segment.
+        }  // Next segment.
+    }  // Next part.
 }
