@@ -1,5 +1,6 @@
 #include "csf.h"
 #include "csfimpl.h"
+#include <string.h>
 
 static void UINT1tLdd(size_t nrCells, void *Buf)
 {
@@ -239,7 +240,7 @@ static void ConvertToREAL4( size_t nrCells, void *buf, CSF_CR src)
 				if ( ((UINT4 *)buf)[i] == MV_UINT4 )
 					((UINT4 *)buf)[i] = MV_UINT4;
 				else
-					((REAL4 *)buf)[i] = ((UINT4 *)buf)[i];
+					((REAL4 *)buf)[i] = (REAL4)((UINT4 *)buf)[i];
 			    }
 			while(i != 0);
 		}
@@ -364,7 +365,7 @@ static CSF_CONV_FUNC ConvFunc(CSF_CR destType, CSF_CR srcType)
 	PRECOND(convTableIndex[CSF_UNIQ_CR_MASK(destType)] != -1);
 	/* don't complain on illegal, it can be attached
 	 * to a app2file while there's no WRITE_MODE
-	 * if it's an error then it's catched in RputSomeCells
+	 * if it's an error then it's cached in RputSomeCells
 	 */
 	return 
          ConvTable[(int)convTableIndex[CSF_UNIQ_CR_MASK(srcType)]]
@@ -437,83 +438,109 @@ int RuseAs(
   CSF_CR inFileCR = RgetCellRepr(m);
   CSF_VS inFileVS = RgetValueScale(m);
   int hasInFileCellReprType2 =  HasInFileCellReprType2(inFileCR);
+  int useTypeNoEnumIn;
+  int useTypeNoEnum;
 
-  switch(useType)
-  {
-    case VS_BOOLEAN:
-  	switch(inFileVS) {
-	   case VS_LDD: case VS_DIRECTION:
-   			M_ERROR(CANT_USE_AS_BOOLEAN);
-   			return 1;
-   	   case VS_BOOLEAN:
-   	   		POSTCOND(inFileCR == CR_UINT1);
-   	   		m->appCR = CR_UINT1;
-   	   		m->file2app = same;
-   	   		m->app2file = same;
-   	   		return 0;
-   	   default:
-			if ( (! hasInFileCellReprType2) && WRITE_ENABLE(m) )
-			{ /* cellrepr is old one, we can't write that */
-					M_ERROR(CANT_USE_WRITE_BOOLEAN);
-					return 1;
-			}
-   			m->appCR = CR_UINT1;
-   			m->file2app  = ConvFuncBool(inFileCR);
-   			m->app2file = ConvFunc(inFileCR, CR_UINT1);
-   	   		return 0;
-   	} /* case useType == VS_BOOLEAN */
-   	break;
+  /* It is very inconvenient that both, VS and CR are taken as arguments
+   * for this function, and previously were used in the switch statement
+   * now, at least 'special conversions' handled first.
+   */
 
-   case VS_LDD:
-        switch(inFileVS) {
-         case VS_LDD:
-   	   		POSTCOND(inFileCR == CR_UINT1);
-   	   		m->appCR = CR_UINT1;
-   	   		m->file2app = same;
-   	   		m->app2file = same;
-   	   		return 0;
-        case VS_CLASSIFIED: 
-        case VS_NOTDETERMINED: 
-        	switch(inFileCR) {
-        	 case CR_UINT1:
-        	 	m->appCR = CR_UINT1;
-			m->file2app  = UINT1tLdd;
-			m->app2file = same;
-			return 0;
-		 case CR_INT2:
-		        if (WRITE_ENABLE(m))
-		        {  M_ERROR(CANT_USE_WRITE_LDD);
-			   return 1;
-		        }
-        	 	m->appCR = CR_UINT1;
-			m->file2app  = INT2tLdd;
-			m->app2file = illegal;
-			return 0;
-                  default:
-                    assert( FALSE );
-		}
-	default: M_ERROR(CANT_USE_AS_LDD);
-		 return 1;
+  /* Hopefully Coverity will no longer detect that useTypeNoEnum */
+  /* comes from useType */
+  useTypeNoEnumIn = useType;
+  memcpy(&useTypeNoEnum, &useTypeNoEnumIn, sizeof(int));
+
+  if(useTypeNoEnum == VS_BOOLEAN){
+    switch(inFileVS) {
+      case VS_LDD:
+      case VS_DIRECTION: {
+        M_ERROR(CANT_USE_AS_BOOLEAN);
+        return 1;
       }
-      /* case useType == VS_LDD */
-      break;
-     case CR_UINT1:
-     case CR_INT4 :
-     case CR_REAL4:
-     case CR_REAL8:
-		if ( (! hasInFileCellReprType2) && WRITE_ENABLE(m) )
-		{ /* cellrepr is old one, we can't write that */
-				M_ERROR(CANT_USE_WRITE_OLDCR);
-				return 1;
-		}
-		m->appCR = useType;
-		m->file2app  = ConvFunc(useType, inFileCR);
-		m->app2file = ConvFunc(inFileCR, useType);
-		POSTCOND(m->file2app != NULL);
-		return 0;
-    default:
-    		M_ERROR(ILLEGAL_USE_TYPE);
-		return 1;
+      case VS_BOOLEAN: {
+        POSTCOND(inFileCR == CR_UINT1);
+        m->appCR = CR_UINT1;
+        m->file2app = same;
+        m->app2file = same;
+        return 0;
+      }
+      default: {
+        if((!hasInFileCellReprType2) && WRITE_ENABLE(m)) {
+          /* cellrepr is old one, we can't write that */
+          M_ERROR(CANT_USE_WRITE_BOOLEAN);
+          return 1;
+        }
+        m->appCR = CR_UINT1;
+        m->file2app  = ConvFuncBool(inFileCR);
+        m->app2file = ConvFunc(inFileCR, CR_UINT1);
+        return 0;
+      }
+    }
+  }
+  else if (useTypeNoEnum == VS_LDD){
+    switch(inFileVS) {
+      case VS_LDD: {
+        POSTCOND(inFileCR == CR_UINT1);
+        m->appCR = CR_UINT1;
+        m->file2app = same;
+        m->app2file = same;
+        return 0;
+      }
+      case VS_CLASSIFIED: 
+      case VS_NOTDETERMINED: {
+        switch(inFileCR) {
+          case CR_UINT1: {
+            m->appCR = CR_UINT1;
+            m->file2app  = UINT1tLdd;
+            m->app2file = same;
+            return 0;
+          }
+          case CR_INT2: {
+            if(WRITE_ENABLE(m)) {
+              M_ERROR(CANT_USE_WRITE_LDD);
+              return 1;
+            }
+            m->appCR = CR_UINT1;
+            m->file2app  = INT2tLdd;
+            m->app2file = illegal;
+            return 0;
+          }
+          default: {
+            /* This should never happen.
+             * Shut up compiler.
+             */
+            assert(0);
+          }
+        }
+      }
+      default: {
+        M_ERROR(CANT_USE_AS_LDD);
+        return 1;
+      }
+    }
+  }
+
+  switch(useType) {
+    case CR_UINT1:
+    case CR_INT4 :
+    case CR_REAL4:
+    case CR_REAL8: {
+      if((!hasInFileCellReprType2) && WRITE_ENABLE(m)) {
+        /* cellrepr is old one, we can't write that */
+        M_ERROR(CANT_USE_WRITE_OLDCR);
+        return 1;
+      }
+      m->appCR = useType;
+      m->file2app  = ConvFunc(useType, inFileCR);
+      m->app2file = ConvFunc(inFileCR, useType);
+      POSTCOND(m->file2app != NULL);
+      return 0;
+    }
+    default: {
+      M_ERROR(ILLEGAL_USE_TYPE);
+      return 1;
+    }
   }
   /* NOTREACHED */
 }
